/*
  Author:  Jonathan Bondzie, Dr. Chuck Yeung
           Lennard Gas Simulation Source

	   Copyright (C) 1999  Penn State University

	   This program is free software; you can redistribute it and/or
	   modify it under the terms of the GNU General Public License
	   as published by the Free Software Foundation;
	   
	   This program is distributed in the hope that it will be useful,
	   but WITHOUT ANY WARRANTY; without even the implied warranty of
	   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	   GNU General Public License for more details.
	   
	   You should have received a copy of the GNU General Public License
	   along with this program; if not, write to the Free Software
	   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
	   02111-1307, USA.

	   email jab341@psu.edu
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define	KISS_RANGE	2147483648.
#define	LKISS_RANGE	4294967296.

/************************************************************************/
/*		The KISS (Keep It Simple Stupid) random			*/
/*		number generator (from Naras)				*/
/*									*/
/*		returns a long int between -2147483648 and 2147483647	*/
/*									*/
/************************************************************************/


/* static variables seeded by seedKiss and used by kiss			*/
/* Naras' seeds xran=123456789, yran=521288629, zran=362436069,wran=2262615; */

static unsigned long xran, yran, zran, wran;

long kiss(void)
{
    xran = 69069 * xran + 1234567;
    yran^=(yran<<13);
    yran^=(yran>>17);
    yran^=(yran<<5);
    zran = 65184 * (zran&65535) + (zran>>16);
    wran = 636638 * (wran&65535) + (wran>>16);
    return(xran + yran + zran + (wran<<16));
}

double lkiss(void)
{
    double	x;

    xran = 69069 * xran + 1234567;
    yran^=(yran<<13);
    yran^=(yran>>17);
    yran^=(yran<<5);
    zran = 65184 * (zran&65535) + (zran>>16);
    wran = 636638 * (wran&65535) + (wran>>16);
    x = (xran + yran + zran + (wran<<16))/LKISS_RANGE; 
    return(x);
}

/*************************************************************************/
/*	seeds the random number generator				 */
/*************************************************************************/

void seedKiss( unsigned long seed1, unsigned long seed2, unsigned long seed3 )

{
	int	i;

	xran = seed1; yran = seed2; zran = seed3;
	for( i = 0; i < 1000; i++ ) kiss();	/* get rid of transients */
}


/************************************************************************/
/* makes a Gaussian distribution using the kiss random number generator */
/* the standard deviation is unity					*/
/************************************************************************/

double gaussD(void)
{

	static	float	gset;
	static	int	iset=0;
        float   v1, v2, r, fac;

        if ( iset == 0 ){
		do {
	             	v1 = kiss()/KISS_RANGE; v2 = kiss()/KISS_RANGE;
                	r = v1*v1 + v2*v2;
		} while( r >= 1 );
		iset = 1;
                fac = sqrt( -2. * log(r)/r );
                gset = v1 * fac;
		v2 = v2*fac;
		return(v2);
        }
	else{
                iset = 0;
		return(gset);
        }
}

